# Change Log

## 3.0.3

### Patch Changes

- de7f7b1a: Fix the typescript typings in testing helpers for projects that depend on `lit` package
- Updated dependencies [de7f7b1a]
  - @open-wc/testing-helpers@2.0.2

## 3.0.2

### Patch Changes

- 70f52431: export chai in index.d.ts

## 3.0.1

### Patch Changes

- 1649ba46: Release bump version as major versions have already been used and unpublished in an accidental publish about a year ago.
- Updated dependencies [1649ba46]
  - @open-wc/testing-helpers@2.0.1

## 3.0.0

### Major Changes

- 689c9ea3: Upgrade to support latest `lit` package.

  - the exports `html` and `unsafeStatic` are now deprecated we recommend to import them directly from `lit/static-html.js`;
  - You need to load a polyfill for the scoped registry if you wanna use the `scopedElements` option
  - We now enforce our entrypoints via an export map
  - The side effect free import got renamed to `pure`

    ```js
    // old
    import { fixture } from '@open-wc/testing-helpers/index-no-side-effects.js';
    // new
    import { fixture } from '@open-wc/testing-helpers/pure';
    ```

- a5a79a25: We now use an es module version of chai from `@esm-bundle/chai`.

### Minor Changes

- b9b11adc: support snapshot testing with Web Test Runner
- 22c4017c: Undo deprecation of the `html` and `unsafeStatic` exports to enable matching lit versions to what is used in fixture.

  A typical testing file looks like this

  ```js
  import { html, fixture } from '@open-wc/testing'; // html will be lit-html 2.x

  it('works for tags', async () => {
    const el = await fixture(
      html`
        <my-el></my-el>
      `,
    );
  });
  ```

  With this export you can combine the usage of lit-html 2.x for the fixture and template rendering in lit-html 1.x

  ```js
  import { html as fixtureHtml, fixture } from '@open-wc/testing'; // fixtureHtml will be lit-html 2.x
  import { html } from 'my-library'; // html will be lit-html 1.x

  it('works for tags', async () => {
    const el = await fixture(fixtureHtml`<my-el></my-el>`);
  });

  it('can be combined', async () => {
    class MyExtension extends LibraryComponent {
      render() {
        // needs to be lit-html 1.x as the library component is using LitElement with lit-html 1.x
        return html`
          <p>...</p>
        `;
      }
    }

    // fixture requires a lit-html 2.x template
    const el = await fixture(fixtureHtml`<my-el></my-el>`);
  });
  ```

  NOTE: If you are using fixture for testing your lit-html 1.x directives then this will no longer work.
  A possible workaround for this is

  ```js
  import { html, fixture } from '@open-wc/testing'; // html will be lit-html 2.x
  import { render, html as html1, fancyDirective } from 'my-library'; // html and render will be lit-html 1.x

  it('is a workaround for directives', async () => {
    const node = document.createElement('div');
    render(html1`<p>Testing ${fancyDirective('output')}</p>`, node);

    // you can either cleanup yourself or use fixture
    const el = await fixture(
      html`
        ${node}
      `,
    );

    expect(el.children[0].innerHTML).toBe('Testing [[output]]');
  });
  ```

### Patch Changes

- 4b9ea6f6: Use lit@2.0 stable based dependencies across the project.
- 40837d10: - use latest axe-core
  - allow not required attributes in role testing
- 945d1d9c: Remove unused dependency on `mocha` as the environment should bring it.
- 89fdfa03: Do not generate chai plugins on user install. Do it only on monorepo install.
- 45c7fcc1: Import scoped registries code dynamically to prevent library consumers that do not leverage this API from being bound to its load order requirements.
- Updated dependencies [b9b11adc]
- Updated dependencies [4b9ea6f6]
- Updated dependencies [689c9ea3]
- Updated dependencies [22c4017c]
- Updated dependencies [b6e868d5]
- Updated dependencies [40837d10]
- Updated dependencies [45c7fcc1]
- Updated dependencies [580ce0ee]
- Updated dependencies [72e67571]
- Updated dependencies [6940a3cb]
  - @open-wc/semantic-dom-diff@0.19.5
  - @open-wc/testing-helpers@2.0.0
  - chai-a11y-axe@1.3.2

## 3.0.0-next.5

### Minor Changes

- 22c4017c: Undo deprecation of the `html` and `unsafeStatic` exports to enable matching lit versions to what is used in fixture.

  A typical testing file looks like this

  ```js
  import { html, fixture } from '@open-wc/testing'; // html will be lit-html 2.x

  it('works for tags', async () => {
    const el = await fixture(
      html`
        <my-el></my-el>
      `,
    );
  });
  ```

  With this export you can combine the usage of lit-html 2.x for the fixture and template rendering in lit-html 1.x

  ```js
  import { html as fixtureHtml, fixture } from '@open-wc/testing'; // fixtureHtml will be lit-html 2.x
  import { html } from 'my-library'; // html will be lit-html 1.x

  it('works for tags', async () => {
    const el = await fixture(fixtureHtml`<my-el></my-el>`);
  });

  it('can be combined', async () => {
    class MyExtension extends LibraryComponent {
      render() {
        // needs to be lit-html 1.x as the library component is using LitElement with lit-html 1.x
        return html`
          <p>...</p>
        `;
      }
    }

    // fixture requires a lit-html 2.x template
    const el = await fixture(fixtureHtml`<my-el></my-el>`);
  });
  ```

  NOTE: If you are using fixture for testing your lit-html 1.x directives then this will no longer work.
  A possible workaround for this is

  ```js
  import { html, fixture } from '@open-wc/testing'; // html will be lit-html 2.x
  import { render, html as html1, fancyDirective } from 'my-library'; // html and render will be lit-html 1.x

  it('is a workaround for directives', async () => {
    const node = document.createElement('div');
    render(html1`<p>Testing ${fancyDirective('output')}</p>`, node);

    // you can either cleanup yourself or use fixture
    const el = await fixture(
      html`
        ${node}
      `,
    );

    expect(el.children[0].innerHTML).toBe('Testing [[output]]');
  });
  ```

### Patch Changes

- Updated dependencies [22c4017c]
  - @open-wc/testing-helpers@2.0.0-next.2

## 3.0.0-next.4

### Patch Changes

- 4b9ea6f6: Use lit@2.0 stable based dependencies across the project.
- 945d1d9c: Remove unused dependency on `mocha` as the environment should bring it.
- 45c7fcc1: Import scoped registries code dynamically to prevent library consumers that do not leverage this API from being bound to its load order requirements.
- Updated dependencies [4b9ea6f6]
- Updated dependencies [45c7fcc1]
  - @open-wc/testing-helpers@2.0.0-next.1

## 3.0.0-next.3

### Patch Changes

- 40837d10: - use latest axe-core
  - allow not required attributes in role testing
- Updated dependencies [40837d10]
- Updated dependencies [580ce0ee]
  - chai-a11y-axe@1.3.2-next.0
  - @open-wc/semantic-dom-diff@0.19.5-next.2

## 3.0.0-next.2

### Minor Changes

- b9b11adc: support snapshot testing with Web Test Runner

### Patch Changes

- Updated dependencies [b9b11adc]
  - @open-wc/semantic-dom-diff@0.19.5-next.0

## 3.0.0-next.1

### Patch Changes

- 89fdfa03: Do not generate chai plugins on user install. Do it only on monorepo install.

## 3.0.0-next.0

### Major Changes

- 689c9ea3: Upgrade to support latest `lit` package.

  - the exports `html` and `unsafeStatic` are now deprecated we recommend to import them directly from `lit/static-html.js`;
  - You need to load a polyfill for the scoped registry if you wanna use the `scopedElements` option
  - We now enforce our entrypoints via an export map
  - The side effect free import got renamed to `pure`

    ```js
    // old
    import { fixture } from '@open-wc/testing-helpers/index-no-side-effects.js';
    // new
    import { fixture } from '@open-wc/testing-helpers/pure';
    ```

- a5a79a25: We now use an es module version of chai from `@esm-bundle/chai`.

### Patch Changes

- Updated dependencies [689c9ea3]
  - @open-wc/testing-helpers@2.0.0-next.0

## 2.5.33

### Patch Changes

- aee0ee63: export waitUntil from index-no-side-effects

## 2.5.32

### Patch Changes

- 0362fe08: Keep hand written types for semantic-dom-diff for now
- Updated dependencies [0362fe08]
  - @open-wc/semantic-dom-diff@0.19.3

## 2.5.31

### Patch Changes

- 4a81d791: Add types folder to npm artifacts
- Updated dependencies [4a81d791]
  - @open-wc/semantic-dom-diff@0.19.2
  - @open-wc/testing-helpers@1.8.12

## 2.5.30

### Patch Changes

- 17e9e7dc: Change type distribution workflow
- Updated dependencies [17e9e7dc]
  - @open-wc/semantic-dom-diff@0.19.1
  - @open-wc/testing-helpers@1.8.11

All notable changes to this project will be documented in this file.
See [Conventional Commits](https://conventionalcommits.org) for commit guidelines.

## [2.5.29](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.28...@open-wc/testing@2.5.29) (2020-10-03)

**Note:** Version bump only for package @open-wc/testing

## [2.5.28](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.27...@open-wc/testing@2.5.28) (2020-10-02)

**Note:** Version bump only for package @open-wc/testing

## [2.5.27](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.26...@open-wc/testing@2.5.27) (2020-09-25)

**Note:** Version bump only for package @open-wc/testing

## [2.5.26](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.25...@open-wc/testing@2.5.26) (2020-09-11)

**Note:** Version bump only for package @open-wc/testing

## [2.5.25](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.24...@open-wc/testing@2.5.25) (2020-08-27)

**Note:** Version bump only for package @open-wc/testing

## [2.5.24](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.23...@open-wc/testing@2.5.24) (2020-08-19)

**Note:** Version bump only for package @open-wc/testing

## [2.5.23](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.22...@open-wc/testing@2.5.23) (2020-08-16)

**Note:** Version bump only for package @open-wc/testing

## [2.5.22](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.21...@open-wc/testing@2.5.22) (2020-08-14)

**Note:** Version bump only for package @open-wc/testing

## [2.5.21](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.20...@open-wc/testing@2.5.21) (2020-08-05)

**Note:** Version bump only for package @open-wc/testing

## [2.5.20](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.19...@open-wc/testing@2.5.20) (2020-08-04)

**Note:** Version bump only for package @open-wc/testing

## [2.5.19](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.18...@open-wc/testing@2.5.19) (2020-07-08)

**Note:** Version bump only for package @open-wc/testing

## [2.5.18](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.17...@open-wc/testing@2.5.18) (2020-06-15)

**Note:** Version bump only for package @open-wc/testing

## [2.5.17](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.16...@open-wc/testing@2.5.17) (2020-05-25)

**Note:** Version bump only for package @open-wc/testing

## [2.5.16](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.15...@open-wc/testing@2.5.16) (2020-05-01)

**Note:** Version bump only for package @open-wc/testing

## [2.5.15](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.14...@open-wc/testing@2.5.15) (2020-04-26)

**Note:** Version bump only for package @open-wc/testing

## [2.5.14](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.13...@open-wc/testing@2.5.14) (2020-04-26)

### Bug Fixes

- **semantic-dom-diff:** allow assertion message ([c8a3b18](https://github.com/open-wc/open-wc/commit/c8a3b18936de43cba0227c146e77b60a04327e41))

## [2.5.13](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.12...@open-wc/testing@2.5.13) (2020-04-21)

### Bug Fixes

- **testing:** export types from chai ([ce4b91a](https://github.com/open-wc/open-wc/commit/ce4b91a9a22ecff48a48c05f35fd19c93ba3bc24))

## [2.5.12](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.11...@open-wc/testing@2.5.12) (2020-04-20)

**Note:** Version bump only for package @open-wc/testing

## [2.5.11](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.10...@open-wc/testing@2.5.11) (2020-04-12)

**Note:** Version bump only for package @open-wc/testing

## [2.5.10](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.9...@open-wc/testing@2.5.10) (2020-04-06)

### Bug Fixes

- **testing:** auto load chai plugin typings ([a895e3a](https://github.com/open-wc/open-wc/commit/a895e3ab2f5dc57026ff55af04c8909969bdc641))

## [2.5.9](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.8...@open-wc/testing@2.5.9) (2020-04-05)

**Note:** Version bump only for package @open-wc/testing

## [2.5.8](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.7...@open-wc/testing@2.5.8) (2020-03-19)

**Note:** Version bump only for package @open-wc/testing

## [2.5.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.6...@open-wc/testing@2.5.7) (2020-03-19)

**Note:** Version bump only for package @open-wc/testing

## [2.5.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.5...@open-wc/testing@2.5.6) (2020-03-10)

**Note:** Version bump only for package @open-wc/testing

## [2.5.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.4...@open-wc/testing@2.5.5) (2020-03-10)

**Note:** Version bump only for package @open-wc/testing

## [2.5.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.3...@open-wc/testing@2.5.4) (2020-02-09)

**Note:** Version bump only for package @open-wc/testing

## [2.5.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.2...@open-wc/testing@2.5.3) (2020-02-06)

### Bug Fixes

- **testing:** export "waitUntil" type ([730514f](https://github.com/open-wc/open-wc/commit/730514f111926d6fb0ea750994e98482bb6ab092))

## [2.5.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.1...@open-wc/testing@2.5.2) (2020-01-31)

### Bug Fixes

- skip brooken published versions ([25d21de](https://github.com/open-wc/open-wc/commit/25d21def522f22f98fc8c71b4c055617089c0e23))

## [2.5.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.5.0...@open-wc/testing@2.5.1) (2020-01-19)

**Note:** Version bump only for package @open-wc/testing

# [2.5.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.4.4...@open-wc/testing@2.5.0) (2020-01-07)

### Features

- **testing-helpers:** add waitUntil helper ([bef5dac](https://github.com/open-wc/open-wc/commit/bef5dac522f8bb0e497d0dd0e24eadbf39b98987))

## [2.4.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.4.3...@open-wc/testing@2.4.4) (2020-01-07)

### Bug Fixes

- **testing:** add missing files to npm ([962fc10](https://github.com/open-wc/open-wc/commit/962fc10f0af17120af8cd64f04509ebe80fc9cdf))

## [2.4.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.4.2...@open-wc/testing@2.4.3) (2020-01-07)

### Bug Fixes

- **testing:** compatibility with webpack ([e355027](https://github.com/open-wc/open-wc/commit/e355027a6a8fb68686d18205bb456a515e86b5f3))

## [2.4.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.4.1...@open-wc/testing@2.4.2) (2019-12-05)

**Note:** Version bump only for package @open-wc/testing

## [2.4.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.4.0...@open-wc/testing@2.4.1) (2019-11-24)

**Note:** Version bump only for package @open-wc/testing

# [2.4.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.9...@open-wc/testing@2.4.0) (2019-11-19)

### Bug Fixes

- **testing:** cut illegal char in snapshot names ([e952abd](https://github.com/open-wc/open-wc/commit/e952abdace3cef6d48d1f48c2285d51fdb67e73e)), closes [#963](https://github.com/open-wc/open-wc/issues/963)

### Features

- update testing to use auto compatibility of es-dev-server ([7d5ea56](https://github.com/open-wc/open-wc/commit/7d5ea56a1cb911fb31228bec18dc38ea8826b1f4))

## [2.3.9](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.8...@open-wc/testing@2.3.9) (2019-11-02)

**Note:** Version bump only for package @open-wc/testing

## [2.3.8](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.7...@open-wc/testing@2.3.8) (2019-10-25)

### Bug Fixes

- align used mocha version ([#901](https://github.com/open-wc/open-wc/issues/901)) ([3606381](https://github.com/open-wc/open-wc/commit/3606381))

## [2.3.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.6...@open-wc/testing@2.3.7) (2019-10-23)

### Bug Fixes

- add package keywords ([#859](https://github.com/open-wc/open-wc/issues/859)) ([cd78405](https://github.com/open-wc/open-wc/commit/cd78405))

## [2.3.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.5...@open-wc/testing@2.3.6) (2019-10-22)

**Note:** Version bump only for package @open-wc/testing

## [2.3.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.4...@open-wc/testing@2.3.5) (2019-10-13)

**Note:** Version bump only for package @open-wc/testing

## [2.3.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.3...@open-wc/testing@2.3.4) (2019-09-15)

**Note:** Version bump only for package @open-wc/testing

## [2.3.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.2...@open-wc/testing@2.3.3) (2019-08-27)

### Bug Fixes

- **testing:** load types for chai plugins ([f122098](https://github.com/open-wc/open-wc/commit/f122098))

## [2.3.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.1...@open-wc/testing@2.3.2) (2019-08-20)

### Bug Fixes

- do not destructure exports to support es-module-lexer ([3709413](https://github.com/open-wc/open-wc/commit/3709413))

## [2.3.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.3.0...@open-wc/testing@2.3.1) (2019-08-18)

### Bug Fixes

- include \*.ts files in npm packages ([8087906](https://github.com/open-wc/open-wc/commit/8087906))

# [2.3.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.8...@open-wc/testing@2.3.0) (2019-08-18)

### Features

- **testing:** use chai instead of @bundled-es-modules/chai ([53579c2](https://github.com/open-wc/open-wc/commit/53579c2))
- add type definition files for testing ([462a29f](https://github.com/open-wc/open-wc/commit/462a29f))

## [2.2.8](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.7...@open-wc/testing@2.2.8) (2019-08-07)

**Note:** Version bump only for package @open-wc/testing

## [2.2.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.6...@open-wc/testing@2.2.7) (2019-08-05)

### Bug Fixes

- cleanup package.json scripts ([be6bdb5](https://github.com/open-wc/open-wc/commit/be6bdb5))

## [2.2.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.5...@open-wc/testing@2.2.6) (2019-08-04)

**Note:** Version bump only for package @open-wc/testing

## [2.2.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.4...@open-wc/testing@2.2.5) (2019-08-04)

**Note:** Version bump only for package @open-wc/testing

## [2.2.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.3...@open-wc/testing@2.2.4) (2019-08-04)

**Note:** Version bump only for package @open-wc/testing

## [2.2.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.2...@open-wc/testing@2.2.3) (2019-08-04)

**Note:** Version bump only for package @open-wc/testing

## [2.2.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.1...@open-wc/testing@2.2.2) (2019-07-30)

**Note:** Version bump only for package @open-wc/testing

## [2.2.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.2.0...@open-wc/testing@2.2.1) (2019-07-28)

**Note:** Version bump only for package @open-wc/testing

# [2.2.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.1.4...@open-wc/testing@2.2.0) (2019-07-27)

### Features

- expose elementUpdated testing-helper by default ([#653](https://github.com/open-wc/open-wc/issues/653)) ([55a165f](https://github.com/open-wc/open-wc/commit/55a165f))

## [2.1.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.1.3...@open-wc/testing@2.1.4) (2019-07-26)

**Note:** Version bump only for package @open-wc/testing

## [2.1.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.1.2...@open-wc/testing@2.1.3) (2019-07-25)

**Note:** Version bump only for package @open-wc/testing

## [2.1.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.1.1...@open-wc/testing@2.1.2) (2019-07-24)

**Note:** Version bump only for package @open-wc/testing

## [2.1.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.1.0...@open-wc/testing@2.1.1) (2019-07-24)

**Note:** Version bump only for package @open-wc/testing

# [2.1.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.7...@open-wc/testing@2.1.0) (2019-07-24)

### Features

- **testing:** adding a11y testing via chai-a11y-axe plugin ([5f05b53](https://github.com/open-wc/open-wc/commit/5f05b53))

## [2.0.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.6...@open-wc/testing@2.0.7) (2019-07-24)

**Note:** Version bump only for package @open-wc/testing

## [2.0.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.5...@open-wc/testing@2.0.6) (2019-07-22)

**Note:** Version bump only for package @open-wc/testing

## [2.0.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.4...@open-wc/testing@2.0.5) (2019-07-22)

**Note:** Version bump only for package @open-wc/testing

## [2.0.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.3...@open-wc/testing@2.0.4) (2019-07-19)

**Note:** Version bump only for package @open-wc/testing

## [2.0.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.2...@open-wc/testing@2.0.3) (2019-07-17)

**Note:** Version bump only for package @open-wc/testing

## [2.0.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.1...@open-wc/testing@2.0.2) (2019-07-17)

**Note:** Version bump only for package @open-wc/testing

## [2.0.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@2.0.0...@open-wc/testing@2.0.1) (2019-07-17)

**Note:** Version bump only for package @open-wc/testing

# [2.0.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.15...@open-wc/testing@2.0.0) (2019-07-16)

### Features

- **testing:** upgrade testing-karma to latest version ([2e1be09](https://github.com/open-wc/open-wc/commit/2e1be09))

### BREAKING CHANGES

- **testing:** Removed the legacy flag which used webpack on older
  browsers. We now use karma-esm everywhere which supports older
  browsers with a compatibility option. For more details please see
  the changelog of testing-karma and karma-esm.

## [1.0.15](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.14...@open-wc/testing@1.0.15) (2019-07-16)

### Bug Fixes

- **testing:** bugfix release to keep non breaking testing-karma version ([99b4905](https://github.com/open-wc/open-wc/commit/99b4905))

## [1.0.14](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.13...@open-wc/testing@1.0.14) (2019-07-15)

### Bug Fixes

- adopt to new testing-karma setup ([bdcc717](https://github.com/open-wc/open-wc/commit/bdcc717))

## [1.0.13](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.12...@open-wc/testing@1.0.13) (2019-07-14)

**Note:** Version bump only for package @open-wc/testing

## [1.0.12](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.11...@open-wc/testing@1.0.12) (2019-07-13)

**Note:** Version bump only for package @open-wc/testing

## [1.0.11](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.10...@open-wc/testing@1.0.11) (2019-07-08)

**Note:** Version bump only for package @open-wc/testing

## [1.0.10](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.9...@open-wc/testing@1.0.10) (2019-07-08)

**Note:** Version bump only for package @open-wc/testing

## [1.0.9](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.8...@open-wc/testing@1.0.9) (2019-07-08)

**Note:** Version bump only for package @open-wc/testing

## [1.0.8](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.7...@open-wc/testing@1.0.8) (2019-07-08)

**Note:** Version bump only for package @open-wc/testing

## [1.0.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.6...@open-wc/testing@1.0.7) (2019-07-08)

**Note:** Version bump only for package @open-wc/testing

## [1.0.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.5...@open-wc/testing@1.0.6) (2019-07-02)

**Note:** Version bump only for package @open-wc/testing

## [1.0.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.4...@open-wc/testing@1.0.5) (2019-07-02)

**Note:** Version bump only for package @open-wc/testing

## [1.0.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.3...@open-wc/testing@1.0.4) (2019-06-30)

**Note:** Version bump only for package @open-wc/testing

## [1.0.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.2...@open-wc/testing@1.0.3) (2019-06-23)

**Note:** Version bump only for package @open-wc/testing

## [1.0.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.1...@open-wc/testing@1.0.2) (2019-06-23)

**Note:** Version bump only for package @open-wc/testing

## [1.0.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@1.0.0...@open-wc/testing@1.0.1) (2019-06-18)

**Note:** Version bump only for package @open-wc/testing

# [1.0.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.6...@open-wc/testing@1.0.0) (2019-06-14)

### Features

- utils and webpack plugin for an index.html entrypoint ([#474](https://github.com/open-wc/open-wc/issues/474)) ([c382cc7](https://github.com/open-wc/open-wc/commit/c382cc7))

### BREAKING CHANGES

- Replaced webpack html plugin with index html plugin

## [0.12.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.5...@open-wc/testing@0.12.6) (2019-06-08)

**Note:** Version bump only for package @open-wc/testing

## [0.12.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.4...@open-wc/testing@0.12.5) (2019-05-25)

**Note:** Version bump only for package @open-wc/testing

## [0.12.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.3...@open-wc/testing@0.12.4) (2019-05-19)

**Note:** Version bump only for package @open-wc/testing

## [0.12.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.2...@open-wc/testing@0.12.3) (2019-05-14)

**Note:** Version bump only for package @open-wc/testing

## [0.12.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.1...@open-wc/testing@0.12.2) (2019-05-14)

**Note:** Version bump only for package @open-wc/testing

## [0.12.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.12.0...@open-wc/testing@0.12.1) (2019-05-06)

**Note:** Version bump only for package @open-wc/testing

# [0.12.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.7...@open-wc/testing@0.12.0) (2019-05-06)

### Features

- update to latest testing-karma config syntax ([465bfe0](https://github.com/open-wc/open-wc/commit/465bfe0))

## [0.11.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.6...@open-wc/testing@0.11.7) (2019-05-03)

**Note:** Version bump only for package @open-wc/testing

## [0.11.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.5...@open-wc/testing@0.11.6) (2019-04-30)

**Note:** Version bump only for package @open-wc/testing

## [0.11.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.4...@open-wc/testing@0.11.5) (2019-04-28)

### Bug Fixes

- **eslint-config:** loosen up rules for test and stories files ([#408](https://github.com/open-wc/open-wc/issues/408)) ([3fd251e](https://github.com/open-wc/open-wc/commit/3fd251e))

## [0.11.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.3...@open-wc/testing@0.11.4) (2019-04-14)

### Bug Fixes

- update generator usage ([5d284d4](https://github.com/open-wc/open-wc/commit/5d284d4))

## [0.11.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.2...@open-wc/testing@0.11.3) (2019-04-13)

**Note:** Version bump only for package @open-wc/testing

## [0.11.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.1...@open-wc/testing@0.11.2) (2019-04-08)

**Note:** Version bump only for package @open-wc/testing

## [0.11.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.11.0...@open-wc/testing@0.11.1) (2019-04-08)

**Note:** Version bump only for package @open-wc/testing

# [0.11.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.12...@open-wc/testing@0.11.0) (2019-04-08)

### Features

- **semantic-dom-diff:** add support for snapshot testing ([f7a675a](https://github.com/open-wc/open-wc/commit/f7a675a))

## [0.10.12](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.11...@open-wc/testing@0.10.12) (2019-04-06)

**Note:** Version bump only for package @open-wc/testing

## [0.10.11](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.10...@open-wc/testing@0.10.11) (2019-04-05)

### Bug Fixes

- do not assume available global types of users ([cd394d9](https://github.com/open-wc/open-wc/commit/cd394d9))

## [0.10.10](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.9...@open-wc/testing@0.10.10) (2019-04-03)

**Note:** Version bump only for package @open-wc/testing

## [0.10.9](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.8...@open-wc/testing@0.10.9) (2019-03-31)

### Bug Fixes

- **semantic-dom-diff:** add get-diffable-html.js to npm bundle ([7ee3ba9](https://github.com/open-wc/open-wc/commit/7ee3ba9))

## [0.10.8](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.7...@open-wc/testing@0.10.8) (2019-03-31)

**Note:** Version bump only for package @open-wc/testing

## [0.10.7](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.6...@open-wc/testing@0.10.7) (2019-03-31)

### Bug Fixes

- **testing:** update instructions for new karma config setup ([2de10bf](https://github.com/open-wc/open-wc/commit/2de10bf))

## [0.10.6](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.5...@open-wc/testing@0.10.6) (2019-03-31)

### Bug Fixes

- adopt new karma setup for all packages ([1888260](https://github.com/open-wc/open-wc/commit/1888260))

## [0.10.5](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.4...@open-wc/testing@0.10.5) (2019-03-29)

**Note:** Version bump only for package @open-wc/testing

## [0.10.4](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.3...@open-wc/testing@0.10.4) (2019-03-28)

**Note:** Version bump only for package @open-wc/testing

## [0.10.3](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.2...@open-wc/testing@0.10.3) (2019-03-27)

**Note:** Version bump only for package @open-wc/testing

## [0.10.2](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.1...@open-wc/testing@0.10.2) (2019-03-24)

### Bug Fixes

- adjust generator-open-wc links to create ([cc014b1](https://github.com/open-wc/open-wc/commit/cc014b1))

## [0.10.1](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.10.0...@open-wc/testing@0.10.1) (2019-03-23)

### Bug Fixes

- do not assume globally setup mocha types ([977d5b4](https://github.com/open-wc/open-wc/commit/977d5b4))

# [0.10.0](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.28...@open-wc/testing@0.10.0) (2019-03-23)

### Features

- add types + linting & improve intellisense ([b6d260c](https://github.com/open-wc/open-wc/commit/b6d260c))

## [0.9.28](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.27...@open-wc/testing@0.9.28) (2019-03-20)

**Note:** Version bump only for package @open-wc/testing

## [0.9.27](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.26...@open-wc/testing@0.9.27) (2019-03-14)

**Note:** Version bump only for package @open-wc/testing

## [0.9.26](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.25...@open-wc/testing@0.9.26) (2019-03-14)

**Note:** Version bump only for package @open-wc/testing

## [0.9.25](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.24...@open-wc/testing@0.9.25) (2019-03-08)

**Note:** Version bump only for package @open-wc/testing

## [0.9.24](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.23...@open-wc/testing@0.9.24) (2019-03-06)

**Note:** Version bump only for package @open-wc/testing

## [0.9.23](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.22...@open-wc/testing@0.9.23) (2019-03-04)

**Note:** Version bump only for package @open-wc/testing

## [0.9.22](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.21...@open-wc/testing@0.9.22) (2019-03-03)

**Note:** Version bump only for package @open-wc/testing

## [0.9.21](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.20...@open-wc/testing@0.9.21) (2019-02-26)

**Note:** Version bump only for package @open-wc/testing

## [0.9.20](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.19...@open-wc/testing@0.9.20) (2019-02-24)

**Note:** Version bump only for package @open-wc/testing

## [0.9.19](https://github.com/open-wc/open-wc/compare/@open-wc/testing@0.9.18...@open-wc/testing@0.9.19) (2019-02-16)

### Bug Fixes

- update package repository fields with monorepo details ([cb1acb7](https://github.com/open-wc/open-wc/commit/cb1acb7))

## [0.9.18](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.17...@open-wc/testing@0.9.18) (2019-02-14)

**Note:** Version bump only for package @open-wc/testing

## [0.9.17](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.16...@open-wc/testing@0.9.17) (2019-02-13)

### Bug Fixes

- **testing:** add info to docs to test legacy browsers ([012a867](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/012a867))

## [0.9.16](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.15...@open-wc/testing@0.9.16) (2019-02-11)

**Note:** Version bump only for package @open-wc/testing

## [0.9.15](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.14...@open-wc/testing@0.9.15) (2019-02-04)

**Note:** Version bump only for package @open-wc/testing

## [0.9.14](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.13...@open-wc/testing@0.9.14) (2019-02-02)

### Bug Fixes

- unify npm readme header for all open-wc packages ([1bac939](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/1bac939))

## [0.9.13](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.12...@open-wc/testing@0.9.13) (2019-02-02)

**Note:** Version bump only for package @open-wc/testing

## [0.9.12](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.11...@open-wc/testing@0.9.12) (2019-01-31)

### Bug Fixes

- **testing:** missing nextFrame export ([101f79b](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/101f79b)), closes [#176](https://github.com/open-wc/open-wc/tree/master/packages/testing/issues/176)

## [0.9.11](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.10...@open-wc/testing@0.9.11) (2019-01-26)

**Note:** Version bump only for package @open-wc/testing

## [0.9.10](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.9...@open-wc/testing@0.9.10) (2019-01-26)

**Note:** Version bump only for package @open-wc/testing

## [0.9.9](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.8...@open-wc/testing@0.9.9) (2019-01-24)

### Bug Fixes

- add docu for fixtureCleanup ([ab0170a](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/ab0170a))

## [0.9.8](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.7...@open-wc/testing@0.9.8) (2019-01-23)

**Note:** Version bump only for package @open-wc/testing

## [0.9.7](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.6...@open-wc/testing@0.9.7) (2019-01-20)

### Bug Fixes

- refactor generators ([1dab1f4](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/1dab1f4))

## [0.9.6](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.5...@open-wc/testing@0.9.6) (2019-01-19)

### Bug Fixes

- move fixtureCleanup to testing helpers ([#136](https://github.com/open-wc/open-wc/tree/master/packages/testing/issues/136)) ([9d268ab](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/9d268ab))

## [0.9.5](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.4...@open-wc/testing@0.9.5) (2019-01-19)

**Note:** Version bump only for package @open-wc/testing

## [0.9.4](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.3...@open-wc/testing@0.9.4) (2019-01-16)

### Bug Fixes

- improve documentation ([4f5472f](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/4f5472f))

## [0.9.3](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.2...@open-wc/testing@0.9.3) (2019-01-13)

**Note:** Version bump only for package @open-wc/testing

## [0.9.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.1...@open-wc/testing@0.9.2) (2019-01-09)

### Bug Fixes

- docu typos ([aafd5e4](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/aafd5e4))

## [0.9.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.9.0...@open-wc/testing@0.9.1) (2019-01-03)

**Note:** Version bump only for package @open-wc/testing

# [0.9.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.6...@open-wc/testing@0.9.0) (2019-01-02)

### Features

- **testing-helpers:** fixture can handle strings and TemplateResults ([0649ea0](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/0649ea0))

## [0.8.6](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.5...@open-wc/testing@0.8.6) (2018-12-29)

**Note:** Version bump only for package @open-wc/testing

## [0.8.5](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.4...@open-wc/testing@0.8.5) (2018-12-23)

**Note:** Version bump only for package @open-wc/testing

## [0.8.4](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.3...@open-wc/testing@0.8.4) (2018-12-22)

**Note:** Version bump only for package @open-wc/testing

## [0.8.3](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.2...@open-wc/testing@0.8.3) (2018-12-22)

**Note:** Version bump only for package @open-wc/testing

## [0.8.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.1...@open-wc/testing@0.8.2) (2018-12-20)

### Bug Fixes

- properly apply prettier ([a12bb09](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/a12bb09))

## [0.8.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.8.0...@open-wc/testing@0.8.1) (2018-12-20)

**Note:** Version bump only for package @open-wc/testing

# [0.8.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.7.2...@open-wc/testing@0.8.0) (2018-12-19)

### Features

- use extendable karma configs by default ([8fd9435](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/8fd9435))

## [0.7.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.7.1...@open-wc/testing@0.7.2) (2018-12-18)

### Bug Fixes

- add testing generator + update docu ([81c765d](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/81c765d))

## [0.7.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.7.0...@open-wc/testing@0.7.1) (2018-12-13)

### Bug Fixes

- apply prettier; add lint-staged ([43acfad](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/43acfad))

# [0.7.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.6.3...@open-wc/testing@0.7.0) (2018-12-11)

### Features

- add typescript type declaration files ([f5cb243](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/f5cb243))

## [0.6.3](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.6.2...@open-wc/testing@0.6.3) (2018-12-02)

**Note:** Version bump only for package @open-wc/testing

## [0.6.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.6.1...@open-wc/testing@0.6.2) (2018-12-01)

**Note:** Version bump only for package @open-wc/testing

## [0.6.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.6.0...@open-wc/testing@0.6.1) (2018-11-30)

### Bug Fixes

- move documentation to READMEs of packages ([b4a0426](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/b4a0426))

# [0.6.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.5.0...@open-wc/testing@0.6.0) (2018-11-26)

### Features

- use latest testing-karma features ([5edc46c](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/5edc46c))

# [0.5.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.4.2...@open-wc/testing@0.5.0) (2018-11-18)

### Features

- sinon is no longer a mandatory package ([ef97cec](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/ef97cec))
- use es module chai version; auto-register side-effects ([263f4ff](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/263f4ff))

## [0.4.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.4.1...@open-wc/testing@0.4.2) (2018-11-18)

**Note:** Version bump only for package @open-wc/testing

## [0.4.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.4.0...@open-wc/testing@0.4.1) (2018-11-16)

**Note:** Version bump only for package @open-wc/testing

# [0.4.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.3.2...@open-wc/testing@0.4.0) (2018-11-15)

### Features

- simplify testing-helpers names ([68e1cb5](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/68e1cb5))

## [0.3.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.3.1...@open-wc/testing@0.3.2) (2018-11-12)

**Note:** Version bump only for package @open-wc/testing

## [0.3.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.3.0...@open-wc/testing@0.3.1) (2018-11-05)

### Bug Fixes

- add karma.conf.js to npmignore ([9700532](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/9700532))

# [0.3.0](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.2.5...@open-wc/testing@0.3.0) (2018-11-05)

### Bug Fixes

- add an npmignore file ([ddceeca](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/ddceeca))

### Features

- **testing:** refactor using testing-helpers, chai-dom-equals ([d6ac78c](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/d6ac78c))

## [0.2.5](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.2.4...@open-wc/testing@0.2.5) (2018-10-28)

### Bug Fixes

- use version ranges ([694e137](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/694e137))

## [0.2.4](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.2.3...@open-wc/testing@0.2.4) (2018-10-27)

### Bug Fixes

- **deps:** update dependency sinon to v7.1.0 ([99a095e](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/99a095e))

## [0.2.3](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.2.2...@open-wc/testing@0.2.3) (2018-10-14)

### Bug Fixes

- **deps:** update dependency sinon to v7 ([a50d8c8](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/a50d8c8))

## [0.2.2](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.2.1...@open-wc/testing@0.2.2) (2018-10-10)

**Note:** Version bump only for package @open-wc/testing

## [0.2.1](https://github.com/open-wc/open-wc/tree/master/packages/testing/compare/@open-wc/testing@0.2.0...@open-wc/testing@0.2.1) (2018-10-07)

**Note:** Version bump only for package @open-wc/testing

# 0.2.0 (2018-10-06)

### Bug Fixes

- **deps:** pin dependencies to 4.2.0 ([98f41f5](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/98f41f5))
- add minimal readme ([9e52ca2](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/9e52ca2))

### Features

- rename test to testing ([d171018](https://github.com/open-wc/open-wc/tree/master/packages/testing/commit/d171018))

<a name="0.1.0"></a>

# 0.1.0 (2018-09-29)

### Features

- initial release of [@open-wc](https://github.com/open-wc)/test ([02b63ab](https://github.com/open-wc/open-wc/tree/master/packages/test/commit/02b63ab))
